local _, L = ...
local mediaFolder = "Interface\\AddOns\\".._.."\\Media\\"

local LSM = LibStub('LibSharedMedia-3.0')

local select = select
local unpack = unpack
local pairs = pairs
local print = print
local math_floor = math.floor

local CreateFrame = CreateFrame
local SetFrameLevel = SetFrameLevel
local ClearAllPoints = ClearAllPoints
local SetPoint = SetPoint
local CreateTexture = CreateTexture
local SetVertexColor = SetVertexColor
local SetStatusBarColor = SetStatusBarColor
local SetBackdropBorderColor = SetBackdropBorderColor
local SetTexture = SetTexture

local C_NamePlate_GetNamePlateForUnit = C_NamePlate.GetNamePlateForUnit
local C_NamePlate_GetNamePlates = C_NamePlate.GetNamePlates
local UnitExists = UnitExists
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitIsUnit = UnitIsUnit
local UnitName = UnitName
local UnitGUID = UnitGUID
local UnitPowerType = UnitPowerType
local UnitStagger = UnitStagger
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local GetRuneCooldown = GetRuneCooldown
local GetUnitChargedPowerPoints = GetUnitChargedPowerPoints
local SetMinMaxValues = SetMinMaxValues
local SetReverseFill = SetReverseFill
local WarlockPowerBar_UnitPower = WarlockPowerBar_UnitPower
local GetPlayerAuraBySpellID = GetPlayerAuraBySpellID

classBarMixin = {}

function classBarMixin:IsTalentActive(spellID)
	local classID = select(3, UnitClass("player"))
	local specIndex = GetSpecialization()
	local specId = GetSpecializationInfoForClassID(classID, specIndex)
	
	local configId = C_ClassTalents.GetActiveConfigID()
	if configId == nil then return end
	local configInfo = C_Traits.GetConfigInfo(configId)
	if configInfo == nil then return end
	for _, treeId in ipairs(configInfo.treeIDs) do
		local nodes = C_Traits.GetTreeNodes(treeId)
		for _, nodeId in ipairs(nodes) do
			local node = C_Traits.GetNodeInfo(configId, nodeId)
			if node and node.ID ~= 0 then
				for idx, talentId in ipairs(node.entryIDs) do
					local entryInfo = C_Traits.GetEntryInfo(configId, talentId)
					local definitionInfo = C_Traits.GetDefinitionInfo(entryInfo.definitionID)
					local spellName = GetSpellInfo(definitionInfo.spellID)
					if definitionInfo.spellID == spellID then
						return true 
					end
				end
			end
		end
		
	end
end

function classBarMixin:ClassBarUpdateRune()
	for runeIndex = 1, 6 do
		local start, duration, runeReady = GetRuneCooldown(runeIndex)
		if not runeReady then
			myClassBar[runeIndex]:SetStatusBarColor(unpack(L.config.player_resource_color_empty))
			if start then
				myClassBar[runeIndex].class_points_max = duration
				myClassBar[runeIndex].duration = GetTime() - start
				myClassBar[runeIndex]:SetScript("OnUpdate", 
					function(self, elapsed)
						self.duration = self.duration + elapsed
						if self.duration >= self.class_points_max or self.duration <= 0 then
							self.value:SetText(nil)
						else
							self.value:SetText(namePlateMixin:FormatTime(self.class_points_max - self.duration))
						end
					end
				)
				myClassBar[runeIndex].value:Show()
			end
		else
			myClassBar[runeIndex]:SetStatusBarColor(unpack(L.config.player_resource_color_full))
			myClassBar[runeIndex]:SetScript("OnUpdate", nil)
			myClassBar[runeIndex].value:SetText(nil)
		end
		myClassBar[runeIndex]:Show()
	end
end

function classBarMixin:ClassBarUpdateKick(guid)
	local i = namePlateMixin.PARTY_TABLE[guid].index
	if not namePlateMixin.PARTY_TABLE[guid].kickSpell then
		myClassBar[i]:Hide()
		return
	end

	myClassBar[i].border.bg:SetVertexColor(namePlateMixin.PARTY_TABLE[guid].classColor.r, namePlateMixin.PARTY_TABLE[guid].classColor.g, namePlateMixin.PARTY_TABLE[guid].classColor.b, 1)
	myClassBar[i]:SetStatusBarColor(namePlateMixin.PARTY_TABLE[guid].classColor.r / 3, namePlateMixin.PARTY_TABLE[guid].classColor.g / 3, namePlateMixin.PARTY_TABLE[guid].classColor.b / 3, 1)
	myClassBar[i]:SetMinMaxValues(0, GetSpellBaseCooldown(namePlateMixin.PARTY_TABLE[guid].kickSpell)/1000)
	myClassBar[i].border.shadow:SetBackdropBorderColor(namePlateMixin.PARTY_TABLE[guid].classColor.r, namePlateMixin.PARTY_TABLE[guid].classColor.g, namePlateMixin.PARTY_TABLE[guid].classColor.b, 1)
	
	if namePlateMixin.PARTY_TABLE[guid].kickCount > 0 and L.config.party_kick_count then
		myClassBar[i].value:SetText(namePlateMixin.PARTY_TABLE[guid].kickCountSucced.." ("..namePlateMixin.PARTY_TABLE[guid].kickCount..")")
		myClassBar[i].value:Show()
	else
		myClassBar[i].value:Hide()
	end
		
	if IsInGroup() then
		myClassBar[i]:Show()
	elseif L.config.party_kick_self then
		myClassBar[i]:Show()
	else
		myClassBar[i]:Hide()
	end
				
	if namePlateMixin.PARTY_TABLE[guid].expirationTime > GetTime() then
		myClassBar[i].border.shadow:Hide()
		myClassBar[i].expirationTime = namePlateMixin.PARTY_TABLE[guid].expirationTime
		myClassBar[i]:SetScript("OnUpdate",
			function(self, elapsed)
				local timeLeft = self.expirationTime - GetTime()
				if timeLeft > 0 then
					self:SetAlpha(L.config.party_kick_alpha)
					PixelUtil.SetStatusBarValue(self, timeLeft)
				else
					self:SetAlpha(1)
					if L.config.party_kick_bars_shadow then 
						self.border.shadow:Show()
					end
					self:SetScript("OnUpdate", nil)
					PixelUtil.SetStatusBarValue(self, 0)
				end
			end
		)
	else
		myClassBar[i]:SetAlpha(1)
		PixelUtil.SetStatusBarValue(myClassBar[i], 0)
		if L.config.party_kick_bars_shadow then
			myClassBar[i].border.shadow:Show()
		else
			myClassBar[i].border.shadow:Hide()
		end
	end
end

function classBarMixin:ClassBarUpdateStagger()
	local currstagger = UnitStagger("player")
	if currstagger then 
		local maxhp = UnitHealthMax("player")
		local percent = currstagger/maxhp
		local color = PowerBarColor[BREWMASTER_POWER_BAR_NAME]

		if (percent > STAGGER_YELLOW_TRANSITION and percent < STAGGER_RED_TRANSITION) then
			color = color[STAGGER_YELLOW_INDEX]
		elseif (percent > STAGGER_RED_TRANSITION) then
			color = color[STAGGER_RED_INDEX]
		else
			color = color[STAGGER_GREEN_INDEX]
		end
	
		myClassBar[1].value:SetText(GetSpellCharges(119582))
		myClassBar[1].value:Show()
		myClassBar[1]:SetStatusBarColor(color.r, color.g, color.b)
		myClassBar[1].border.bg:SetVertexColor(color.r / 3, color.g / 3, color.b / 3)
		myClassBar[1]:SetMinMaxValues(0, maxhp)
		myClassBar[1]:Show()
		
		PixelUtil.SetStatusBarValue(myClassBar[1], currstagger)
	end
end

function classBarMixin:ClassBarUpdateCP()
	local class_points_current, class_points_max = UnitPower('player', L.config.playersInfo[L.selfClass][L.selfSpec].enumPowerType), UnitPowerMax('player', L.config.playersInfo[L.selfClass][L.selfSpec].enumPowerType)
	local chargedPowerPoints = GetUnitChargedPowerPoints("player")
	local chargedPowerPointIndex = chargedPowerPoints and chargedPowerPoints[1] or 0
	
	for i = 1, class_points_max do
		if (i <= class_points_current) then
			if class_points_current == class_points_max then
				myClassBar[i]:SetStatusBarColor(unpack(L.config.player_resource_color[6]))
				if L.config.player_resource_animate_shake then
					myClassBar.ShakeAnim:Play()
				end
			else
				myClassBar[i]:SetStatusBarColor(unpack(L.config.player_resource_color[i]))
			end
			if chargedPowerPointIndex == i then
				myClassBar[i]:SetStatusBarColor( 1, .5, 0, 1)
			end
			PixelUtil.SetStatusBarValue(myClassBar[i], 1)
			myClassBar[i]:Show()
			if L.config.player_resource_animate_cp and i == class_points_current and not myClassBar[i].scaleAnim:IsPlaying() then
				myClassBar[i].scaleAnim:Play()
			end
		else
			if (myClassBar[i].scaleAnim:IsPlaying()) then
				myClassBar[i].scaleAnim:Stop()
			end
			myClassBar[i]:Hide()
		end
	end
	
	if L.selfClass == "WARLOCK" and L.selfSpec == 3 then
		local shardPower = WarlockPowerBar:UnitPower('player')
		if (shardPower - class_points_current) > 0 then
			myClassBar[class_points_current + 1]:Show()
			myClassBar[class_points_current + 1]:SetStatusBarColor(unpack(L.config.player_resource_color[class_points_current + 1]))
			PixelUtil.SetStatusBarValue(myClassBar[class_points_current + 1], shardPower - class_points_current)
		end
	end
	
	if L.config.player_resource_sound_enable then
		if (class_points_current >= class_points_max) and not myClassBar.sound_played then
			myClassBar.sound_played = true
			PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.player_resource_sound), L.config.sound_channel)
		elseif (class_points_current < class_points_max) then
			myClassBar.sound_played = false
		end
	end
end

function classBarMixin:ClassBarUpdateMaelstorm()
	local maelstorm_aura = C_UnitAuras.GetPlayerAuraBySpellID(344179)
	local maelstorm_count = maelstorm_aura and maelstorm_aura.applications or 0
	local hailstorm_aura = C_UnitAuras.GetPlayerAuraBySpellID(334196)
	local hailstorm_count = hailstorm_aura and hailstorm_aura.applications or 0
	
	local class_points_max = 10
	if maelstorm_count then
		for i = 1, class_points_max do
			if (i <= maelstorm_count) then
				if hailstorm_count == 0 and maelstorm_count > 4 then 
					myClassBar[i]:SetStatusBarColor(unpack(L.config.player_resource_color[6]))
				else
					myClassBar[i]:SetStatusBarColor(unpack(L.config.player_resource_color[1]))
				end
				myClassBar[i]:Show()
				if L.config.player_resource_animate_cp and i == maelstorm_count and not myClassBar[i].scaleAnim:IsPlaying() then
					myClassBar[i].scaleAnim:Play()
				end
			else
				if (myClassBar[i].scaleAnim:IsPlaying()) then
					myClassBar[i].scaleAnim:Stop()
				end
				myClassBar[i]:Hide()
			end
		end
	end
	
	if L.config.player_resource_sound_enable then
		if maelstorm_count >= 5 and hailstorm_count == 0 and not myClassBar.sound_played then
			myClassBar.sound_played = true
			PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.player_resource_sound), L.config.sound_channel)
			if L.config.player_resource_animate_shake then
				myClassBar.ShakeAnim:Play()
			end
		elseif maelstorm_count < 5 or hailstorm_count > 0 then
			myClassBar.sound_played = false
		end
	end
end

function classBarMixin:ClassBarUpdateWhirlwind()
	local whirlwind_aura = C_UnitAuras.GetPlayerAuraBySpellID(85739)
	local whirlwind_count = whirlwind_aura and whirlwind_aura.applications or 0
	
	local class_points_max = myClassBar.maxBars
	if whirlwind_count then
		for i = 1, class_points_max do
			if (i <= whirlwind_count and not myClassBar[i]:IsShown()) then
				myClassBar[i]:SetStatusBarColor(unpack(L.config.player_resource_color[1]))
				myClassBar[i]:Show()
				if L.config.player_resource_animate_cp and i == whirlwind_count and not myClassBar[i].scaleAnim:IsPlaying() then
					myClassBar[i].scaleAnim:Play()
				end
			elseif i > whirlwind_count then
				if (myClassBar[i].scaleAnim:IsPlaying()) then
					myClassBar[i].scaleAnim:Stop()
				end
				myClassBar[i]:Hide()
			end
		end
	end
	
	if L.config.player_resource_sound_enable then
		if whirlwind_count == 0 and not myClassBar.sound_played then
			myClassBar.sound_played = true
			PlaySoundFile(LSM:Fetch(LSM.MediaType.SOUND, L.config.player_resource_sound), L.config.sound_channel)
			if L.config.player_resource_animate_shake then
				myClassBar.ShakeAnim:Play()
			end
		elseif whirlwind_count > 0 then
			myClassBar.sound_played = false
		end
	end
end

function classBarMixin:ClassBarUpdateHB(event, unit, guid)
	local unitGUID = guid and guid or UnitGUID(unit)
	if (event == "PLAYER_ENTERING_WORLD" or event == "GROUP_ROSTER_UPDATE") then
		myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index]:SetStatusBarColor(namePlateMixin.PARTY_TABLE[guid].classColor.r, namePlateMixin.PARTY_TABLE[guid].classColor.g, namePlateMixin.PARTY_TABLE[guid].classColor.b, namePlateMixin.PARTY_TABLE[guid].classColor.a)
		myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index].border.bg:SetVertexColor(namePlateMixin.PARTY_TABLE[guid].classColor.r / 3, namePlateMixin.PARTY_TABLE[guid].classColor.g / 3, namePlateMixin.PARTY_TABLE[guid].classColor.b / 3, namePlateMixin.PARTY_TABLE[guid].classColor.a)
		myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index]:SetAlpha(1)
		myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index]:Show()
		
	elseif (event == "UNIT_AURA" and L.config.playersInfo[L.selfClass][L.selfSpec].classHasDispell) then
		local showBorderShadow = false
		for index = 1, DEBUFF_MAX_DISPLAY do
			local name = UnitDebuff(unit, index, 'RAID')
			if name then
				showBorderShadow = true
				break
			end
		end
		for tableGUID in pairs(namePlateMixin.PARTY_TABLE) do
			if tableGUID == unitGUID then
				if showBorderShadow then
					namePlateMixin:SetMyBorderColor(myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index].border, 0.2, 0.8, 0.2, 1)
					myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index].border.shadow:Show()
				else
					namePlateMixin:SetMyBorderColor(myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index].border, 0, 0, 0, 1)
					myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index].border.shadow:Hide()
				end
			end
		end
	
	elseif (event == "UNIT_HEALTH") then
		for tableGUID in pairs(namePlateMixin.PARTY_TABLE) do
			if tableGUID == unitGUID then
				local perc = UnitHealth(unit) / UnitHealthMax(unit)
				if L.config.party_health_bars_dynamic_size then
					myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index]:SetSize(myClassBar.tmpSize, L.config.player_resource_position_height[L.config.player_resource_pine] * (2 - perc))
				else
					myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index]:SetSize(myClassBar.tmpSize, L.config.player_resource_position_height[L.config.player_resource_pine])
				end
				PixelUtil.SetStatusBarValue(myClassBar[namePlateMixin.PARTY_TABLE[unitGUID].index], perc)
			end
		end
	
	end
end

function classBarMixin:ClassBarUpdatePosition()
	if L.config.player_resource_pine == "screen" then
		myClassBar:ClearAllPoints()
		myClassBar:SetPoint("CENTER", UIParent, "CENTER", 0, L.config.player_resource_position_y[L.config.player_resource_pine])
		if myClassBar.show then
			myClassBar:Show()
		end
	else
		local myClassBarPlate = C_NamePlate_GetNamePlateForUnit(L.config.player_resource_pine, issecure())
		if myClassBarPlate and not ((myClassBarPlate.UnitFrame.isSelfPlayer or myClassBarPlate.UnitFrame.unitType == "friendly") and L.config.player_resource_pine == "target") then
			myClassBar:ClearAllPoints()
			myClassBar:SetPoint("BOTTOM", myClassBarPlate.UnitFrame.healthBar, "TOP", 0, L.config.player_resource_position_y[L.config.player_resource_pine]) -- + (myClassBarPlate.UnitFrame.powerBar:IsShown() and L.config.powerbar_height or 0))
			myClassBar:SetFrameLevel(myClassBarPlate:GetFrameLevel())
			if myClassBar.show then
				myClassBar:Show()
			end
		else
			myClassBar:Hide()
		end
	end
end

function classBarMixin:UpdateStatusBars(val, texture, fill)
	myClassBar:SetFrameLevel(1)
	myClassBar.maxBars = val
	myClassBar.sound_played = true
	myClassBar:SetSize( L.config.player_resource_position_width[L.config.player_resource_pine], L.config.player_resource_position_height[L.config.player_resource_pine])
	myClassBar.tmpSize = math_floor((L.config.player_resource_position_width[L.config.player_resource_pine]-((myClassBar.maxBars - 1)* L.config.player_resource_position_space[L.config.player_resource_pine])) / myClassBar.maxBars)
	
	for i = 1, val do
		myClassBar[i] = myClassBar[i] or CreateFrame("StatusBar", "myClassBar"..i, myClassBar, "FrameFlashTemplate")
		myClassBar[i]:SetScript("OnUpdate", nil)
		if i <= myClassBar.maxBars then
			myClassBar[i].border = myClassBar[i].border or namePlateMixin:CreateMyBackdrop(myClassBar[i], 1, 0, 0, 0, 1, LSM:Fetch(LSM.MediaType.STATUSBAR, texture), 0, 0, 0, 1, 6, 0, 1, 0, 0.8)
			myClassBar[i].border.shadow:Hide()
			
			myClassBar[i].value = myClassBar[i].value or namePlateMixin:CreateMyText(myClassBar[i], L.config.player_resource_font, L.config.player_resource_fontsize, "OUTLINE", "OVERLAY", "CENTER", "CENTER", false)
			myClassBar[i].value:SetFont(LSM:Fetch(LSM.MediaType.FONT, L.config.player_resource_font), L.config.player_resource_fontsize , L.config.player_resource_font_flag)
			myClassBar[i].value:SetTextColor(unpack(L.config.player_resource_font_color))
			myClassBar[i].value:Hide()
		
			myClassBar[i]:SetStatusBarTexture(LSM:Fetch(LSM.MediaType.STATUSBAR, texture))
			myClassBar[i].border.bg:SetTexture(LSM:Fetch(LSM.MediaType.STATUSBAR, texture))
			myClassBar[i].border.bg:SetVertexColor(0, 0, 0)
			
			myClassBar[i]:SetMinMaxValues(0, 1)
			myClassBar[i]:SetReverseFill(fill)
			PixelUtil.SetStatusBarValue(myClassBar[i], 1)
			myClassBar[i]:SetAlpha(1)
						
			PixelUtil.SetSize(myClassBar[i], myClassBar.tmpSize, L.config.player_resource_position_height[L.config.player_resource_pine])
						
			if i == 1 then
				PixelUtil.SetPoint(myClassBar[i], "BOTTOMLEFT", myClassBar, "BOTTOMLEFT", 0, 0)
			else
				PixelUtil.SetPoint(myClassBar[i], "LEFT", myClassBar[i-1], "RIGHT", L.config.player_resource_position_space[L.config.player_resource_pine], 0)
			end
		end
		myClassBar[i]:Hide()
	end
end

function classBarMixin:ClassBarUpdateEvents(who)
	myClassBar.type = L.config.player_resource_type
	--print(who)
	
	for _, event in pairs({
		"PLAYER_ENTERING_WORLD",
		"NAME_PLATE_UNIT_ADDED",
		"NAME_PLATE_UNIT_REMOVED",
		"PLAYER_TARGET_CHANGED",
		"RUNE_POWER_UPDATE",
		"UNIT_HEALTH",
		"UNIT_AURA",
		"UNIT_POWER_UPDATE",
		"UNIT_DISPLAYPOWER",
		"GROUP_ROSTER_UPDATE",
	}) do myClassBar:UnregisterEvent(event) end
	myClassBar:SetScript("OnUpdate", nil)
	
	if L.config.player_resource_type == 0 then
		myClassBar:Hide()
		myClassBar.show = false
		
	elseif L.config.player_resource_type == 1 and L.selfClass == "DEATHKNIGHT" then
		myClassBar:RegisterEvent("RUNE_POWER_UPDATE")
		myClassBar.show = true
					
	elseif L.config.player_resource_type == 1 and L.selfClass == "MONK" and L.selfSpec == SPEC_MONK_BREWMASTER then
		myClassBar:RegisterEvent("UNIT_HEALTH")
		myClassBar.show = true

	elseif L.config.player_resource_type == 1 and ((L.selfClass == "SHAMAN" and L.selfSpec == 2) or (L.selfClass == "WARRIOR" and L.selfSpec == 2)) then
		myClassBar:RegisterEvent("UNIT_AURA")
		myClassBar.show = true
		
	elseif L.config.player_resource_type == 1 and namePlateMixin:MultiCheck(L.selfClass, "WARLOCK", "PALADIN", "MONK", "MAGE", "ROGUE", "DRUID", "EVOKER") 
		and L.config.playersInfo[L.selfClass][L.selfSpec].powerToken then
		
		if L.selfClass == "DRUID" then
			myClassBar:RegisterUnitEvent("UNIT_DISPLAYPOWER", "player")
		end
		myClassBar:RegisterUnitEvent("UNIT_POWER_UPDATE", "player")
		myClassBar.show = true
			
	elseif L.config.player_resource_type == 2 then
		myClassBar:RegisterEvent("GROUP_ROSTER_UPDATE")
		myClassBar:RegisterEvent("UNIT_HEALTH")
		if L.config.playersInfo[L.selfClass][L.selfSpec].classHasDispell then
			myClassBar:RegisterEvent("UNIT_AURA")
		end
		myClassBar.show = true
		
	elseif L.config.player_resource_type == 3 then
		myClassBar:RegisterEvent("GROUP_ROSTER_UPDATE")
		myClassBar.show = true
	
	else
		myClassBar:SetScript("OnUpdate", nil)
		myClassBar:Hide()
		myClassBar.show = false
	end

	if myClassBar.show then
		for _, event in pairs({
			"PLAYER_ENTERING_WORLD",
			"NAME_PLATE_UNIT_ADDED",
			"NAME_PLATE_UNIT_REMOVED",
			"PLAYER_TARGET_CHANGED",
		}) do myClassBar:RegisterEvent(event) end
	end
end


local function OnClassBarUpdate(myClassBar, event, unit, powerType)
	if myClassBar.type ~= L.config.player_resource_type or event == "MY_CLASS_BAR_REFRESH" then
		classBarMixin:ClassBarUpdateEvents("OnClassBarUpdate type ~= player_resource_type")
	end
	
	if (event == "MY_CLASS_BAR_REFRESH") then
		classBarMixin:ClassBarUpdatePosition()
	end
	
	if L.config.player_resource_type and L.config.player_resource_type > 0 then
	
		if (event == "NAME_PLATE_UNIT_ADDED" or event == "PLAYER_TARGET_CHANGED") then
			if (event == "NAME_PLATE_UNIT_ADDED" and L.config.player_resource_pine == "player" and UnitIsUnit(unit, "player"))
			or (event == "NAME_PLATE_UNIT_ADDED" and L.config.player_resource_pine == "target" and UnitIsUnit(unit, "target"))
			or (event == "PLAYER_TARGET_CHANGED" and L.config.player_resource_pine == "target")
			or L.config.player_resource_pine == "screen" then
				classBarMixin:ClassBarUpdatePosition()
			end
		
		elseif (event == "NAME_PLATE_UNIT_REMOVED") then
			if (L.config.player_resource_pine == "player" and UnitIsUnit(unit, "player")) or (L.config.player_resource_pine == "target" and UnitIsUnit(unit, "target")) then
				myClassBar:Hide()
			end
			
		elseif event == "PLAYER_TALENT_UPDATE" then
			myClassBar.show = false
			myClassBar:Hide()
			classBarMixin:ClassBarUpdateEvents("OnClassBarUpdate PLAYER_TALENT_UPDATE")
						
		elseif L.config.player_resource_type == 1 and L.selfClass == "DEATHKNIGHT" then
			if  (event == "PLAYER_ENTERING_WORLD" or event == "RUNE_POWER_UPDATE") then
				classBarMixin:ClassBarUpdateRune()
				myClassBar.show = true
			elseif (event == "MY_CLASS_BAR_REFRESH") then
				classBarMixin:UpdateStatusBars(6, L.config.player_resource_texture, false)
				classBarMixin:ClassBarUpdateRune()
			end
	
		elseif L.config.player_resource_type == 1 and L.selfClass == "MONK" and L.selfSpec == SPEC_MONK_BREWMASTER then
			if (event == "UNIT_HEALTH" and unit == "player") then
				classBarMixin:ClassBarUpdateStagger()
				myClassBar.show = true
			elseif (event == "MY_CLASS_BAR_REFRESH") then
				classBarMixin:UpdateStatusBars(1, L.config.player_resource_texture, false)
				classBarMixin:ClassBarUpdateStagger()
			end
			
		elseif L.config.player_resource_type == 1 and L.selfClass == "SHAMAN" and L.selfSpec == 2 then
			if (event == "UNIT_AURA" and unit == "player") then
				classBarMixin:ClassBarUpdateMaelstorm()
				myClassBar.show = true
			elseif (event == "MY_CLASS_BAR_REFRESH") then
				classBarMixin:UpdateStatusBars(10, L.config.player_resource_texture, false)
				classBarMixin:ClassBarUpdateMaelstorm()
			end
		
		elseif L.config.player_resource_type == 1 and L.selfClass == "WARRIOR" and L.selfSpec == 2 then
			if (event == "UNIT_AURA" and unit == "player") then
				classBarMixin:ClassBarUpdateWhirlwind()
				myClassBar.show = true
			elseif (event == "MY_CLASS_BAR_REFRESH") then
				classBarMixin:UpdateStatusBars(4, L.config.player_resource_texture, false)
				classBarMixin:ClassBarUpdateWhirlwind()
			end
			
		elseif L.config.player_resource_type == 1 and (namePlateMixin:MultiCheck(L.selfClass, "WARLOCK", "PALADIN", "MONK", "MAGE", "ROGUE", "DRUID", "EVOKER"))
			and L.config.playersInfo[L.selfClass][L.selfSpec].powerToken then
			
			if (event == "PLAYER_ENTERING_WORLD" or (event == "UNIT_POWER_UPDATE" and unit == "player" and powerType == L.config.playersInfo[L.selfClass][L.selfSpec].powerToken)) then
				classBarMixin:ClassBarUpdateCP()
				myClassBar.show = true
			elseif (event == "MY_CLASS_BAR_REFRESH") then
				classBarMixin:UpdateStatusBars(UnitPowerMax('player', L.config.playersInfo[L.selfClass][L.selfSpec].enumPowerType), L.config.player_resource_texture, false)
				classBarMixin:ClassBarUpdateCP()
			end
	
		elseif L.config.player_resource_type == 2 then
			if (event == "PLAYER_ENTERING_WORLD" or event == "GROUP_ROSTER_UPDATE") then
				myClassBar.show = true
			
			elseif (event == "UNIT_HEALTH" or event == "UNIT_AURA") and (UnitInParty(unit) or unit == "player") then
				classBarMixin:ClassBarUpdateHB(event, unit)
				myClassBar.show = true
			
			elseif (event == "MY_CLASS_BAR_REFRESH") then
				classBarMixin:UpdateStatusBars(MAX_PARTY_MEMBERS + 1, L.config.player_resource_texture, false)
				for guid in pairs(namePlateMixin.PARTY_TABLE) do
					classBarMixin:ClassBarUpdateHB("GROUP_ROSTER_UPDATE", nil, guid)
				end
			end
			
		elseif L.config.player_resource_type == 3 then
			if (event == "PLAYER_ENTERING_WORLD" or event == "GROUP_ROSTER_UPDATE") then
				myClassBar.show = true
				
			elseif (event == "MY_CLASS_BAR_REFRESH") then
				classBarMixin:UpdateStatusBars(MAX_PARTY_MEMBERS + 1, L.config.player_resource_texture, true)
				for guid in pairs(namePlateMixin.PARTY_TABLE) do
					classBarMixin:ClassBarUpdateKick(guid)
				end
			end
			
		elseif L.config.player_resource_type == 1 and event == "UNIT_DISPLAYPOWER" then
			if L.selfClass == "DRUID" then
				local powerType, powerToken =  UnitPowerType("player")
				if (powerType == Enum.PowerType.Energy) then
					myClassBar:Show()
				else
					myClassBar:Hide()
				end
			else
				myClassBar:Show()
			end
		end
	else		
		myClassBar:Hide()
	end
end
			

function classBarMixin:UpdateClassBar()
	OnClassBarUpdate(myClassBar, "MY_CLASS_BAR_REFRESH")
end

function classBarMixin:OnEvent(event, ...)
	if ( event == "ADDON_LOADED" ) then
		local name = ...
		if name == _ then
			classBarMixin:UpdateStatusBars(1, L.config.player_resource_texture, false)
		
			if IsAddOnLoaded(_) then
				myClassBar:SetScript("OnEvent", OnClassBarUpdate)
				myClassBar:RegisterEvent("PLAYER_TALENT_UPDATE")
			
				self:UnregisterEvent('ADDON_LOADED')
				self.ADDON_LOADED = nil
			end
		end
	end
end

function classBarMixin:OnLoad()
	for _, event in pairs({
		"ADDON_LOADED",
	}) do self:RegisterEvent(event) end
end
